package com.ningdatech.pmapi.sys.controller;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ningdatech.basic.auth.AbstractLoginUserUtil;
import com.ningdatech.log.annotation.WebLog;
import com.ningdatech.pmapi.common.utils.TreeUtil;
import com.ningdatech.pmapi.sys.entity.Menu;
import com.ningdatech.pmapi.sys.entity.dto.MenuQueryDTO;
import com.ningdatech.pmapi.sys.entity.dto.MenuSaveDTO;
import com.ningdatech.pmapi.sys.entity.dto.MenuUpdateDTO;
import com.ningdatech.pmapi.sys.entity.vo.MenuRoleVO;
import com.ningdatech.pmapi.sys.entity.vo.MenuVO;
import com.ningdatech.pmapi.sys.manage.MenuManage;
import com.ningdatech.pmapi.sys.service.MenuService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * 菜单
 * </p>
 *
 * @author PoffyZhang
 * @since 2022-09-30
 */
@Slf4j
@Validated
@RestController
@RequiredArgsConstructor
@RequestMapping("/api/v1/menu")
@Api(value = "Menu", tags = "系统管理-菜单")
public class MenuController {

    private final MenuService menuService;
    private final MenuManage menuManage;

    /**
     * 查询系统中所有的的菜单树结构， 不用缓存，因为该接口很少会使用，就算使用，也会管理员维护菜单时使用
     */
    @ApiOperation(value = "查询系统所有的菜单", notes = "查询系统所有的菜单")
    @GetMapping("/list")
    public List<Menu> allTree(@Valid @RequestParam(required = false, value = "title") String title) {
        List<Menu> list = menuService.list(Wrappers.lambdaQuery(Menu.class).orderByAsc(Menu::getSort));
        return TreeUtil.buildTree(list, title);
    }

    @ApiOperation(value = "查询当前登录用户的菜单", notes = "查询当前登录用户的菜单")
    @GetMapping("/current_user_menu")
    public List<MenuRoleVO> currentUserMenu() {
        List<Menu> list = menuService.list(Wrappers.lambdaQuery(Menu.class).orderByAsc(Menu::getSort));
        // TODO:登录用户信息
        return menuManage.buildUserMenu(list, null);
    }

    /**
     * 查询某个菜单
     *
     * @param id 主键id
     * @return 查询结果
     */
    @ApiOperation(value = "查询菜单", notes = "查询菜单")
    @GetMapping("/details")
    public MenuQueryDTO getDetails(@RequestParam Long id) {
        Menu menu = menuService.getById(id);
        MenuQueryDTO query = BeanUtil.toBean(menu, MenuQueryDTO.class);
        return query;
    }

    @ApiOperation(value = "保存新菜单", notes = "保存新菜单")
    @PostMapping("/save")
    @WebLog("保存菜单")
    public MenuVO handlerSave(@Valid @RequestBody MenuSaveDTO data) {
        // todo:创建人ID
        menuService.save(data, 0L);
        return BeanUtil.toBean(data, MenuVO.class);
    }

    @ApiOperation(value = "编辑菜单", notes = "编辑菜单")
    @PostMapping("/modify")
    public MenuVO handlerUpdate(@RequestBody MenuUpdateDTO data) {
        menuService.update(data, AbstractLoginUserUtil.getUserId());
        return BeanUtil.toBean(data, MenuVO.class);
    }

    @ApiOperation(value = "删除菜单", notes = "删除菜单")
    @PostMapping("/remove")
    public Boolean handlerDelete(@RequestBody List<Long> ids) {
        return menuService.removeByIdWithCache(ids);
    }

}
